// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI32SSC1M/savedata/savedata.c $
// $Rev: 35161 $
// $Date: 2016-06-06 16:28:07 -0500 (Mon, 06 Jun 2016) $

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "main.h"

#define MAX_INFO_FILE_LENGTH 500


// #defines *******************************************************************

#define	_1M		(1024L * 1024L)



// variables *******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(int fd, s32* chans)
{
	s32		active	= -1;
	char	buf[64];
	int		errs	= 0;
	s32		first	= -1;
	s32		last	= -1;
	s32		single	= -1;

	gsc_label("Input Channels");

	active	= -1;
	errs	+= ai32ssc1m_dsl_ioctl(fd, AI32SSC1M_IOCTL_CHAN_ACTIVE,	&active	);
	errs	+= ai32ssc1m_dsl_ioctl(fd, AI32SSC1M_IOCTL_CHAN_FIRST,	&first	);
	errs	+= ai32ssc1m_dsl_ioctl(fd, AI32SSC1M_IOCTL_CHAN_LAST,	&last	);
	errs	+= ai32ssc1m_dsl_ioctl(fd, AI32SSC1M_IOCTL_CHAN_SINGLE,	&single	);

	if (errs == 0)
	{
		switch (active)
		{
			default:

				errs++;
				printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);
				break;

			case AI32SSC1M_CHAN_ACTIVE_SINGLE:

				chans[0]	= 1;
				sprintf(buf, "# %ld", (long) single);
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_1:

				chans[0]	= 2;
				strcpy(buf, "#s 0-1");
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_3:

				chans[0]	= 4;
				strcpy(buf, "#s 0-3");
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_7:

				chans[0]	= 8;
				strcpy(buf, "#s 0-7");
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_15:

				chans[0]	= 16;
				strcpy(buf, "#s 0-15");
				break;

			case AI32SSC1M_CHAN_ACTIVE_0_31:

				chans[0]	= 32;
				strcpy(buf, "#s 0-31");
				break;

			case AI32SSC1M_CHAN_ACTIVE_RANGE:

				chans[0]	= last - first + 1;

				if (first == last)
					sprintf(buf, "# %ld", (long) first);
				else
					sprintf(buf, "#s %ld-%ld", (long) first, (long) last);

				break;
		}

		if (errs == 0)
		{
			printf(	"%ld Channel%s  (%s)\n",
					(long) chans[0],
					(chans[0] == 1) ? "" : "s",
					buf);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(int fd)
{
	int		errs;
	long	get		= sizeof(_buffer) / 4;
	int		got;

	gsc_label("Reading");
	got	= ai32ssc1m_dsl_read(fd, _buffer, get);

	if (got < 0)
	{
		errs	= 1;
	}
	else if (got != get)
	{
		errs	= 1;
		printf(	"FAIL <---  (got %ld samples, requested %ld)\n",
				(long) got,
				(long) get);
	}
	else
	{
		errs	= 0;
		printf(	"PASS  (%ld samples)\n",
				(long) get);
	}

	return(errs);
}




//*****************************************************************************
static int _save_data_with_datename(int fd, int chan, int acquisition_rate, int errs)
{
  //  char	name_hex[50];
  //  char	name_dec[50];
        char	name[50];

        s32 first_channel=-1;
        s32 last_channel=-1;
        s32 amplitude=-1;
	FILE*		file;
	int			i;
	long		l;
	//	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;
  
	time_t t = time(NULL);
	struct tm tm = *localtime(&t);

	sprintf(name,"data_CLIO_%d-%02d-%02d_%02d-%02d-%02d.txt", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);

 	errs	+= ai32ssc1m_chan_first			(fd, -1, -1, &first_channel);
 	errs	+= ai32ssc1m_chan_last			(fd, -1, -1, &last_channel);
		errs	+= ai32ssc1m_ain_range(fd, -1, -1, &amplitude);
	gsc_label("Filename");
	printf("%s \n",name);

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		fprintf(file,"#date= %d-%02d-%02d_%02d-%02d-%02d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
		fprintf(file,"#first= %d\n", first_channel);
		fprintf(file,"#last= %d\n", last_channel);
		fprintf(file,"#acquisition_rate= %d\n", acquisition_rate);
		fprintf(file,"#amplitude= %d\n", amplitude);


		//		printf("SAMPLES %d \n",samples);
		for (l = 0; l < samples; l++)
		{
			i	= fprintf(file, "%08lX  ", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
			//			if ((l) && (_buffer[l] & 0x80000000))
			if (l%32==31)
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}
			
		}

		fprintf(file,"\r\n");
		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);

} // _save_data_with_name

//*****************************************************************************
static int _save_data(int fd, int chan, int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			if ((l) && (_buffer[l] & 0x80000000))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}

			i	= fprintf(file, "  %08lX", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int save_data(int fd, s32 io_mode)
{
  	s32	chans	= AI32SSC1M_CHAN_ACTIVE_0_31;
	//	s32	chans	= AI32SSC1M_CHAN_ACTIVE_0_15;
	int	errs	= 0;
	s32     timeout = 20;
	s32     acquisition_rate    = 36; // 36=1Ms/s min: 549s/s
	s32     previous_acquisition_rate=0;
	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	//Acquisition rate	
	errs	+= ai32ssc1m_config_ai(fd, -1, acquisition_rate);
	previous_acquisition_rate=acquisition_rate;
	//	errs	+= ai32ssc1m_config_ai(fd, -1, 1440);

	//Trigger:
	//AI32SSC1M_BURST_SYNC_EXT => external trigger
	//AI32SSC1M_BURST_SYNC_DISABLE => Internal trigger
		errs    += ai32ssc1m_burst_sync(fd, -1, AI32SSC1M_BURST_SYNC_DISABLE, NULL); // Internal trigger
	//	errs    += ai32ssc1m_burst_sync(fd, -1, AI32SSC1M_BURST_SYNC_EXT, NULL); // External trigger
	//errs    += ai32ssc1m_burst_sync(fd, -1, AI32SSC1M_BURST_SYNC_RBG, NULL); // External trigger

			//	errs	+= ai32ssc1m_aux_clk_mode(fd,-1,AI32SSC1M_AUX_CLK_MODE_DISABLE,NULL);
		//			errs	+= ai32ssc1m_aux_sync_mode(fd,-1,AI32SSC1M_AUX_SYNC_MODE_INPUT,NULL);
				//errs	+= ai32ssc1m_aux_sync_mode(fd,-1,AI32SSC1M_AUX_SYNC_MODE_DISABLE,NULL);
	errs	+= ai32ssc1m_aux_clk_mode(fd,-1,AI32SSC1M_AUX_CLK_MODE_DISABLE,NULL);
 	errs	+= ai32ssc1m_burst_size(fd, -1, 10240, NULL);

		//		errs    += ai32ssc1m_burst_size(fd, -1, 16, NULL); // Burst size
		
	//Timeout (duration set above)
	errs	+= ai32ssc1m_rx_io_timeout(fd,-1, -1, timeout, NULL);
	
	errs	+= ai32ssc1m_rx_io_mode(fd, -1, io_mode, NULL);
	errs	+= _channels(fd, &chans);

	//Channels range
       	errs	+= ai32ssc1m_chan_active		(fd, -1, AI32SSC1M_CHAN_ACTIVE_RANGE,  NULL);
	errs	+= ai32ssc1m_chan_first			(fd, -1, 0, NULL);
	errs	+= ai32ssc1m_chan_last			(fd, -1, 31,	NULL);
	errs	+= ai32ssc1m_ain_range(fd, -1, AI32SSC1M_AIN_RANGE_5V, NULL);
	//	errs	+= ai32ssc1m_ain_mode(fd, -1, -1, &timeout);
	errs	+= ai32ssc1m_ain_mode(fd, -1, AI32SSC1M_AIN_MODE_DIFF, NULL);
	//	errs	+= ai32ssc1m_ain_mode(fd, -1, AI32SSC1M_AIN_MODE_VREF, NULL);
	gsc_label("1 second before data taking...");
	sleep(1);
	printf("Now\n");
	gsc_label_level_inc();
	for(;;) {
	  gsc_label("Acquiring");
	  errs	+= ai32ssc1m_ain_buf_clear(fd, -1);
	  errs	+= _read_data(fd);
	  errs	+= ai32ssc1m_ain_buf_overflow(fd, -1, -1, NULL);
	  errs	+= ai32ssc1m_ain_buf_underflow(fd, -1, -1, NULL);
	//	errs	+= ai32ssc1m_ain_buf_thr_lvl(fd, -1, -1, &timeout);
	//printf("thr lvl %d \n",timeout);
	  errs	+= _save_data_with_datename(fd, chans, acquisition_rate, errs);
	  printf("Done\n");
	
	  char fileinfo[]="/home/etalon-admin/acquisition_data/acquisition_info.txt";
	  FILE* fid;
	  char line[MAX_INFO_FILE_LENGTH];
	  int pause_between_acquisition_seconds=1;
	  fid  = fopen(fileinfo, "r");
	  if (fid == NULL) {
	      printf("FAIL <---  (unable to read %s)\n", fileinfo);
	  } else {	    
	    fread(line,sizeof(char),MAX_INFO_FILE_LENGTH,fid);
	    printf("Line read: %s\n",line); 
	    int iline=0;
	    while(line[iline]!='\0'){
	      int char_found=0;
	      if (strncmp("PAUSE",&(line[iline]),5)==0) {
		iline+=6;
		pause_between_acquisition_seconds=0;
		while(((line[iline]-'0')>=0)&&((line[iline]-'0')<='9')) {
		  pause_between_acquisition_seconds*=10;
		  pause_between_acquisition_seconds+=(line[iline]-'0');
		  iline++;
		}
		char_found=1;
	      } //line is pause
	      if (strncmp("RATE",&(line[iline]),4)==0) {
		acquisition_rate=0;
		iline+=5;
		while(((line[iline]-'0')>=0)&&((line[iline]-'0')<='9')) {
		  //		  printf("val= %d %c %d %d\n",iline, line[iline], line[iline]-'0', acquisition_rate);
		  acquisition_rate*=10;
		  acquisition_rate+=(line[iline]-'0');
		  iline++;
		}
		if (acquisition_rate!=previous_acquisition_rate) {
		  errs	+= ai32ssc1m_config_ai(fd, -1, acquisition_rate);
		  previous_acquisition_rate=acquisition_rate;
		}
		char_found=1;
	      }
	      if ((line[iline]=='\f')||(line[iline]=='\n')){
		iline++;
		char_found=1;
	      } //special caracters
	      //printf("Reading... %d %d %d %s\n", iline, line[iline],char_found,&(line[iline]));
	      if(char_found==0){
		iline++;
	      }
	    } //while reading line
	    fclose(fid);
	  }
	  printf("Sleep %d\n",pause_between_acquisition_seconds);
	  sleep(pause_between_acquisition_seconds);
	  
	  char filename[]="/home/etalon-admin/acquisition_data/pause_acquisition";
	  int isleep=0;
	  while( access( filename, F_OK ) != -1 ) {
	    // file exists
	    if (isleep==0) {
	      printf("File %s exists\nPause...\n",filename);
	      isleep=1;
	    }
	    printf(".");
	    sleep(10);
	  }
	  char filename_stop[]="/home/etalon-admin/acquisition_data/stop_acquisition";
	  if( access( filename_stop, F_OK ) != -1 ) {
	    printf("File %s is present, exiting...\n",filename_stop);
	    // file exists
	    break; //we leave the for loop
	  }
	}
	gsc_label_level_dec();
	gsc_label_level_dec();
	return(errs);
}



